/**************************************************************************/
/*                                                                        */
/*   2-d Q-state Potts model using Metropolis, Swendsen-Wang, or Wolff    */
/*   Monte Carlo update algorithms.                                       */
/*                                                                        */
/*   Paul Coddington, May 1992                                            */
/*                                                                        */
/**************************************************************************/

/*       
		 FILES USED IN THE PROGRAM:

Prand.c			Simple random number generator
SWmeasure.c		Measurements for Swendsen-Wang update
SWupdate.c		Swendsen-Wang update
Wmeasure.c		Measurements for Wolff update
Wupdate.c		Wolff update
anal.c			Data analysis routines for averages and correlations
aves.c			Backup and restore averages and correlations
clusters.c              Identifies and labels the clusters
dataflow.c              Backup and restore the lattice of spins
init.c                  Initialize spins
main.c                  Main control program
measure.c               Do measurements of energy and magnetization
metmeasure.c            Measure acceptance for Metropolis update
metupdate.c             Metropolis update
print.c                 Print out a part of the lattice of spins
printdata.c             Print out the data at the end of the run
rng.c                   Good random number generator (uses Prand to initialize)
writedata.c             Write out results of latest measurement
*/

#include "comext.h"
#include "comdef.h"
#include <stdio.h>
#include <string.h>
#include <math.h>


main()
{
   char ctemp,avesname[128],lastavesname[128],datatype[128];
   char outname[128];
   int i,j,l,d,iset,wsweep,swps,iflag,jflag,xflag;
   short changed;
   FILE *faves,*fd,*fopen();

   anal = FALSE;
   initialized = FALSE;
   frustrated = FALSE;
   ferromag = FALSE;
   glass = FALSE;

   powerof2[0] = 1; 
   for (i=1; i<LEVELS; i++) powerof2[i] = 2 * powerof2[i-1]; 

   magsq[1] = 0.0;
   for (i=0;i<MAXNS;i++) {
      ns[i] = 0;
   }

   /* Read in parameters */

   printf("Output file format (ASCII or BINARY)? ");
   scanf("%s", datatype);
   printf("%s\n", datatype);
   if (strncmp("ASCII",datatype,1) == 0) {
     ASCII = TRUE; BINARY = FALSE;
   }
   else if (strncmp("BINARY",datatype,1) == 0) {
     ASCII = FALSE; BINARY = TRUE;
   }
   else {
     perror("data type not specified");
     exit(-2);
   }

   /* Number of Potts states */
   printf("Q? ");
   scanf("%d",&Q);
   printf("%d\n",Q);

   /* Coupling (inverse temperature) */
   printf("beta? "); 
   scanf("%f",&beta);
   printf("%12.8f\n",beta);

   /* Lattice size */
   printf("x,y? ");
   scanf("%d %d",&isiz,&jsiz);
   printf("%d %d\n",isiz,jsiz);
   sites = isiz*jsiz;
   printf("sites: %d\n",sites);

   /* Set up constant J[i][j] and P[i][j]. These can be changed 
      later if necessary. */
   if (beta == 0.0) {  /* Take this to mean zero temperature */
      p = 1.0;
   }
   else if (beta < 0.0) {
      printf("beta must be >= 0.0\n");
      fflush(stdout);
      perror("beta must be >= 0.0");
      exit(-3);
   }
   else {
      p = 1 - exp(-(double)beta);
   }
   printf("p = %12.8f\n",p);
   for (i=0;i<isiz;i++) {
   for (j=0;j<jsiz;j++) {
      for (d=0;d<DIMENSION;d++) {
	 J[i][j][d] = 1.0;
	 P[i][j][d] = p;
      }
   }}

   printf("output filename extension? ");
   scanf("%s",fname) ;
   printf("%s\n",fname) ;

   printf("first run? ");
   scanf("%d", &first);
   printf("%d\n",first);
   fflush(stdout);

   /* Create an output file (only used if BINARY) */
   sprintf(outname, "bout.%s", fname);
   if ((fd=fopen(outname,"w")) == NULL) {
      perror("open") ;
      exit( -1 ) ;
   }

   if (BINARY) {
     cfwrite(&Q,sizeof(int),1,fd);
     cfwrite(&beta,sizeof(float),1,fd);
     cfwrite(&isiz,sizeof(int),1,fd);
     cfwrite(&jsiz,sizeof(int),1,fd);
   }

   volume = (double)sites;
   two_beta = 2.0 * beta;

   printf("Enter a menu option (type '?' for a help):\n");
   fflush(stdout);

   /* Read in menu options */
   while ((ctemp=getchar()) != 'Q') {

     switch(ctemp) {

       case 'i' : /* Initialize spin configuration */
		  printf("Start: cold, hot or from file (0,1,2)? ") ;
	          scanf("%d",&iflag);
	          printf("%d\n",iflag);
	          if (iflag == 2) {
		     printf("Read from file ? ") ;
	             scanf("%s",lastfname) ;
	             printf("back.%s\n",lastfname);
		  }
		  fflush(stdout);
		  init(spin,iflag);
		  if ((iflag == 2) && (BINARY))
		     cfwrite(&randx,sizeof(long),1,fd);
		  initialized = TRUE;
	          break;

       case 'j' : /* Initialize couplings */
		  printf("Set couplings: 0 = constant ferromagnetic\n");
                  printf("               1 = constant anti-ferromagnetic\n");
                  printf("               2 = read from file\n");
                  printf("               3 = random ferromagnetic\n");
                  printf("               4 = random anti-ferromagnetic\n");
                  printf("               5 = +/- J spin glass\n");
                  printf("               6 = gaussian spin glass\n");
                  printf("               7 = fully frustrated Ising\n");
                  printf("               8 = dilute fully frustrated Ising\n");
                  printf("Couplings? ");
	          scanf("%d",&jflag) ;
	          printf("%d\n",jflag);
	          jinit(jflag) ;
		  fflush(stdout);
	          break;

       case 'a' : /* Do data analysis during the run */
		  printf("bin size? ");
		  scanf("%d", &binsz);
		  printf("%d\n", binsz);
		  printf("Maximum displacement for autocorr measurements? ");
		  scanf("%d", &NA);
		  printf("%d\n", NA);
                  anal = TRUE;
		  break;

       case 'u' : /* Update */
		  if (!initialized) {
		    perror("not initialized yet");
		    exit( -3 );
		  }
	          printf("number of bins? ") ;
		  scanf("%d",&sets) ;
		  printf("%d\n",sets) ;
		  printf("measurements per bin (bin size)? ") ;
		  scanf("%d",&wsweeps) ;
		  printf("%d\n",wsweeps) ;
		  printf("sweeps per measurement? ") ;
		  scanf("%d",&sweeps) ;
		  printf("%d\n",sweeps) ;
		  num_iterations = sweeps * sites;
		  if (BINARY) {
		    cfwrite(&sets,sizeof(int),1,fd);
		    swps = wsweeps * sweeps;
		    cfwrite(&swps,sizeof(int),1,fd);
		  }
		  if (anal) {
		     numbins = sets * wsweeps / binsz;
		     analinit();
                  }
		  fflush(stdout);
		  
		  for (iset=0;iset<sets;iset++) {
		     for (wsweep=0;wsweep<wsweeps;wsweep++) {
#ifdef METROPOLIS
			if (beta == 0.0)  /* Zero temperature */
			   metupdate_zero(sweeps);
			else
		           metupdate(sweeps);
		        measure(spin);
		        metmeasure();
#endif METROPOLIS
#ifdef SW
		        SWupdate(sweeps);
		        measure(spin);
		        SWmeasure();
#endif SW
#ifdef WOLFF
		        Wupdate(sweeps);
		        measure(spin);
		        Wmeasure();
#endif WOLFF
			if (anal) {
			   analdata();
			}
		     }
		     writedata(fd);
		  }
		  totswps += sets*wsweeps*sweeps;
		  break;

       case 'm' : /* Do a measurement */
                  if (BINARY) { /* Change so that it prints out ASCII */
		     ASCII = TRUE; 
		     changed = TRUE;
		  }
		  else changed = FALSE;
		  measure(spin);
		  writedata(fd);
		  if (changed)  /* Change back to binary */
		     ASCII = FALSE;
		  break;

       case 's':  /* Set the seed for the random number generator */
		  setseed() ;
                  /* Set up the seeds for the TWC rng as well */
                  init_seeds();
		  break;

       case 'b':  /* backup the configuration(s) */
		  backup(spin);
		  break;

       case 'p':  /* Print out the spins */
		  print(spin,0);
		  break;

       case '?' : printf(" i : initialise \n");
		  printf(" u : update # of times \n");
		  printf(" a : analyze data during the run\n");
		  printf(" m : measure energy and total spin \n"); 
          	  printf(" s : set the seed of the rng\n") ;
		  printf(" b : backup the lattice \n");
		  printf(" p : print the lattice \n");
		  printf(" j : set up couplings J[i][j][dir] \n");
		  printf(" Q : Quit \n"); 
		  printf(" ? : for this list \n"); 
		  break;

     } /* end switch ctemp */

   } /* end while getchar */

   /* Print out the averages */
   if (anal) {
      sprintf(avesname, "aves.%s", fname);
      if ((faves = fopen(avesname, "w")) == NULL) {
         perror("aves open") ;
         exit( -1 ) ;
      }
      printaves(faves);
      fclose(faves);
   }

   printf("totswps: %d\n", totswps);
   printf("END\n") ;
   if (BINARY) cfwrite(&totswps,sizeof(int),1,fd);

   /* Print out analyzed data*/
   if (anal) {
      printdata(fname);
   }

   exit( 0 ) ;

} /* end main */


